
/* ***************************  Definitions  ************************** */
DEFINE BUFFER b-ca-u_shipment        FOR ca-u_shipment.
DEFINE BUFFER b-ca-u_booking         FOR ca-u_booking.
DEF VAR iNumPart AS INT    NO-UNDO.

/* ***************************  Main Block  *************************** */

/*******************************************************************************************/
/* Van het bookingnr veld het numerieke deel opslaan IN bookingnr_num_part                 */
/* naar aanleiding van call 90320-982 tbv Eucon                                            */
/* de write trigger is aangepast, nu hoeven we alleen maar de trigger te activeren om de   */
/* nieuwe index en het nieuwe veld te vullen                                               */
/*******************************************************************************************/

/*-- slechts eenmaal uitvoeren --*/
IF CAN-FIND(FIRST sp-control WHERE
                  sp-control.controlkode EQ 'ca-patch-01-fill-booking-num-part'
               OR sp-control.controlkode EQ 'ca-after-01-fill-booking-num-part' )
THEN RETURN.

DISABLE TRIGGERS FOR LOAD OF ca-u_shipment.
DISABLE TRIGGERS FOR LOAD OF ca-u_booking.

FOR EACH ca-u_line_parameters WHERE
         ca-u_line_parameters.bknr_seqnr_seqno GT 0 OR
         ca-u_line_parameters.bknr_year_seqno  GT 0
         NO-LOCK:

   shipment:
   FOR EACH ca-u_shipment WHERE
            ca-u_shipment.companycode EQ ca-u_line_parameters.companycode AND
            ca-u_shipment.linecode    EQ ca-u_line_parameters.linecode
            NO-LOCK:
      DO TRANSACTION:
         FIND b-ca-u_shipment WHERE
              ROWID(b-ca-u_shipment) EQ ROWID(ca-u_shipment)
              EXCLUSIVE-LOCK.
         RUN caGetNumPart (INPUT b-ca-u_shipment.bookingnr, OUTPUT iNumPart).
         ASSIGN b-ca-u_shipment.bookingnr_num_part = STRING(iNumPart).
      END. /*transaction*/
   END. /*each shipment*/

   booking:
   FOR EACH ca-u_booking WHERE
            ca-u_booking.companycode EQ ca-u_line_parameters.companycode AND
            ca-u_booking.linecode    EQ ca-u_line_parameters.linecode
            NO-LOCK:
      DO TRANSACTION:
         FIND b-ca-u_booking WHERE
              ROWID(b-ca-u_booking) EQ ROWID(ca-u_booking)
              EXCLUSIVE-LOCK.
         RUN caGetNumPart (INPUT b-ca-u_booking.bookingnr, OUTPUT iNumPart).
         ASSIGN b-ca-u_booking.bookingnr_num_part = STRING(iNumPart).
      END. /*transaction*/
   END. /*each booking*/

END. /*each line parameters*/

/*-- registreren voor toekomstige runs --*/
DO TRANSACTION:
   CREATE sp-control.
   ASSIGN sp-control.controlkode = 'ca-patch-01-fill-booking-num-part'
          sp-control.lijst       = 'uitgevoerd op ' + STRING(TODAY,'99-99-9999') + ' ' + STRING(TIME,'HH:MM:SS')
          sp-control.uitleg      = 'bijwerken van veld bookingnr_num_part in tabellen ca-u_shipment & ca-u_booking'.
END.

PROCEDURE caGetNumPart.
   DEF INPUT  PARAMETER pcBookingNr AS CHAR   NO-UNDO.
   DEF OUTPUT PARAMETER piNumPart   AS INT    NO-UNDO.
   DEFINE VARIABLE cPart             AS CHARACTER  NO-UNDO.
   DEFINE VARIABLE iBknr             AS INTEGER    NO-UNDO.
   DEFINE VARIABLE iStart            AS INTEGER    NO-UNDO INITIAL 1. /*1=offset waarde*/
   DEFINE VARIABLE iNumPartLength    AS INTEGER    NO-UNDO.
   DEFINE VARIABLE iStretch          AS INTEGER    NO-UNDO.

   /*-- hub, hub, hub, barba truuk:-----------------------------------------
     Omdat er gekozen was voor een integer num_part, en het bookingsnr
     dynamisch gevuld KAN worden (of door vrije ingave gebruiker) dienen
     we het getal van num_part voorspelbaar te maken om tijdens de query
     een range te kunnen bepalen; dit doen we door het getal 'op te rekken'
     naar de volledige lengte van het bookingnr volgens ca-u_line_parameters
     dus bijvoorbeeld 90001 blijft 90001 (year 1, seqno 4), maar 000001 wordt
     dus volgens dezelfde lineparameter definitie (1+4=5) 10000. Als nu de
     gebruiker in het zoekveld opgeeft 901, dan zoeken we in de range
     901000 t/m 901999, maar is alleen 1 ingetypt, dan zoeken we in de range
     100000 t/m 199999, en dan wordt record met nummer 00001 ook gevonden. Anders
     gezegd, als de gebruiker 1 intoetst, dan wil die alle records zien waarbij
     het year+seqno deel begint met 1. Ja ik weet het, character veld met
     BEGINS doet dat ook. Maar de integer keuze was een al een feit. Nadeel is
     ook dat de integer maar maximaal 1,4E9 mag zijn, Dus effectief kunnen we
     maar tot 9E8 opslaan, is de combo year + seqno groter (bv 4+6=10) dan 8,
     dan slaan we het domweg niet meer op. en is het vette pech voor gebruiker
     Er staat een call in spTrag om een db wijziging door te voeren om te
     kunnen zoeken binnen het bookingnr.
     ----------------------------------------------------------------------*/

   DO iBknr = 1 TO 7: /*-- begin positie bereken door de hele opbouw van bookingnr te doorlopen (iStart) --*/
      CASE iBknr:
         WHEN ca-u_line_parameters.bknr_seqnr_seqno      THEN ASSIGN cPart          = cPart          + SUBSTRING( pcBookingNr, iStart, ca-u_line_parameters.bknr_seqnr_length )
                                                                     iNumPartLength = iNumPartLength + ca-u_line_parameters.bknr_seqnr_length
                                                                     iStart         = iStart         + ca-u_line_parameters.bknr_seqnr_length.
         WHEN ca-u_line_parameters.bknr_year_seqno       THEN ASSIGN cPart          = cPart          + SUBSTRING( pcBookingNr, iStart, ca-u_line_parameters.bknr_year_length )
                                                                     iNumPartLength = iNumPartLength + ca-u_line_parameters.bknr_year_length
                                                                     iStart         = iStart         + ca-u_line_parameters.bknr_year_length.
         WHEN ca-u_line_parameters.bknr_pod_seqno        THEN ASSIGN iStart         = iStart         + ca-u_line_parameters.bknr_pod_length.
         WHEN ca-u_line_parameters.bknr_pol_seqno        THEN ASSIGN iStart         = iStart         + ca-u_line_parameters.bknr_pol_length.
         WHEN ca-u_line_parameters.bknr_scac_code_seqno  THEN ASSIGN iStart         = iStart         + ca-u_line_parameters.bknr_scac_code_length.
         WHEN ca-u_line_parameters.bknr_text_seqno       THEN ASSIGN iStart         = iStart         + LENGTH( ca-u_line_parameters.bknr_text ).
         WHEN ca-u_line_parameters.bknr_voyage_seqno     THEN ASSIGN iStart         = iStart         + ca-u_line_parameters.bknr_voyage_length.
      END CASE.
   END. /*do iBknr*/

   /*-- omzetten naar integer --*/
   /* change_bknr:  0=Never -> bookingnr disabled
                    1=Only for cross-booking -> sensitive = bookingagentnr eq bookingagentnr in lineparameters
                    2=No restriction -> bookingnr enabled
                    3=Cross-booking and on copy -> sensitive = bookingagentnr eq bookingagentnr in lineparameters
   */
   IF /* ca-u_line_parameters.change_bknr NE '2' AND /*is niet handmatig ingevuld*/ */
      iNumPartLength LE 8 /*er was gekozen voor integer en maximale waarde die een integer kan opslaan is 1,4E9*/
   THEN DO:
      ASSIGN piNumPart = INTEGER( cPart ) NO-ERROR.
      /*-- en oprekken --*/
      DO iStretch =  LENGTH(STRING( piNumPart )) + 1 TO iNumPartLength:
         piNumPart = piNumPart * 10.
      END.
   END.
   /* indien bovenstaande niet wordt uitgevoerd, dan blijft piNumPart=0, dit reset dan
      het bookingnr_num_part. Dit is o.a. nodig omdat de conversie ook gebruik maakt
      van deze trigger */
END PROCEDURE.
